import xml.etree.ElementTree as ET
from xml.etree.ElementTree import Element

from .types import DocumentTokens
from .utils import HREF, strip_whitespace

def join_tokens(tokens: DocumentTokens) -> str:
    """
    Convert fb2 xml to html, joins bodies into one string
    """

    res = b""

    xml_root = ET.fromstring(strip_whitespace(tokens["content"]))
    for body in xml_root.iterfind("./body"):
        res += process_section(body, tokens)

    return res


def process_section(body: Element, tokens: DocumentTokens) -> str:
    """
    Processes individual sections, recursively goes throw sections tree
    """

    res = b"<section>\n"

    for tag_name in ("title", "epigraph", "annotation"):
        tag = body.find("./" + tag_name)
        if tag:
            process_content(tag, tokens)
            res += children_to_html(tag)
    image = body.find("./image")
    if image:
        process_image(image, tokens)
        res += ET.tostring(image)

    for section in body.findall("./section"):
        if section.find("./section"):
            res += process_section(section, tokens)
        else:
            process_content(section, tokens)
            res += b"<section>\n" + children_to_html(section) + b"</section>\n"

    return res + b"</section>\n"


def children_to_html(root: Element) -> str:
    """
    Converts xml tag children to string
    """

    res = b""

    for child in root:
        res += ET.tostring(child)

    return res


def process_image(element: Element, tokens: DocumentTokens) -> None:
    r"""
    Converts fb2 \<image /\> to html \<img /\>. Replaces xlink:href with src="\<base64_image_data\>"
    """

    element.tag = "img"

    href = element.get(HREF)
    element.attrib.pop(HREF)

    element.set("src", tokens[href[1:]] if href[0] == "#" else href)


tag_replacement = {
    "empty-line": "br",
    "emphasis": "em",
    "strikethrough": "strike",
    "v": "p",
}

tag_with_class = {
    "subtitle": "p",
    "cite": "div",
    "poem": "div",
    "stanza": "div",
    "epigraph": "div",
    "text-author": "p",
}


def process_content(root: Element, tokens: DocumentTokens) -> None:
    """
    Converts fb2 xml tag names to html equivalents and my own styled elements.
    Resolves binary data dependencies
    """

    for child in root:
        process_content(child, tokens)

        if child.tag == "a":
            href = child.get(HREF)
            child.attrib.pop(HREF)
            child.set("href", href)

        if child.tag == "image":
            process_image(child, tokens)

        elif child.tag in tag_replacement.keys():
            child.tag = tag_replacement[child.tag]

        elif child.tag in tag_with_class.keys():
            child.set("class", child.tag)
            child.tag = tag_with_class[child.tag]
